'use strict';
/*global $,window,Aviary,_ */

var btn = {
    'delete': function () {
        return $('<button type="button" class="nf_listfield_delete"><i class="icon-remove"></i></button>')
            .click(function (e) {
                e.stopPropagation();
                e.preventDefault();

                var t = $(this);
                t.parent().slideUp(400, function () {
                    $(this).remove();
                });
                updateListfield(t.closest('.nf_listfield_container'));
                var self = $(this).closest('.nf_listfield').data('listfield');
                self.length--;
            });
    },
    drag: function () {
        return $('<div class="nf_listfield_drag"><i title="Drag to reorder" class="icon-resize-vertical"></i></div>');
    },
    add: function () {
        return $('<button type="button" class="nf_add btn btn-warning"><i class="icon-plus icon-white"></i> Add Item</button>');
    }
};


function initFieldSet(ctx) {
    // On first run, close all field-sets, open only those with errors
    if (!ctx) {
        $('.error').closest('.nf_fieldset > div, .nf_listfield_container > div').show();
    }

    $('.nf_fieldset, .nf_listfield_container', ctx).each(function () {
        if ($(this).data('nf_fieldset')) {
            return;
        }
        $(this).data('nf_fieldset', true);

        var $this = $(this),
            isTopLevel = $this.is('.toplevel'),
            h2 = $('> h2, > label', $this),
            div = $('> div', $this),
            i = $('<i class="icon-chevron-right" />').prependTo($this);

        $this.find('label').add(i).add(h2).off('click').on('click', function chevronToggle(e) {
            e.stopPropagation();
            toggleVisibility();
        });

        function toggleVisibility() {
            $this.is_open = !$this.hasClass('closed');
            var divs = isTopLevel ? $this.find('div') : div;
            var im = isTopLevel ? $this.find('i.icon-chevron-down, i.icon-chevron-right') : i;
            if ($this.is_open) {
                im.removeClass('icon-chevron-down').addClass('icon-chevron-right');
                divs.stop(1, 1).slideUp('fast');
                $this.addClass('closed');
            } else {
                im.removeClass('icon-chevron-right').addClass('icon-chevron-down');
                divs.stop(1, 1).slideDown('fast');
                $this.removeClass('closed');
            }
        }

        toggleVisibility();
    });
}


function updateListfield(t) {
    var length = $('> .nf_listfield > ul > li', t).length - 1;
    t.find('.list_summary').text(length ? length + ' items' : 'No items');
    return length;
}


function initWidgets(ctx) {
    $('.nf_listfield', ctx).each(function () {
        $(this).data('listfield', new ListField(this));
    });
    initFieldSet(ctx);
    if ($.fn.select2) {
        $('.socket-select', ctx).each(getSocketFunctionForSelect2);
        $('.nf_ref', ctx).each(getQueryFunctionForSelect2);
        $('select', ctx).select2();
    }
    if ($.fn.datepicker) $('.nf_datepicker', ctx).datepicker({format: 'yyyy-mm-dd'});
    $('[data-ref]', ctx).each(refLink);

    // Wire FilePicker widget. FieldBinding is done automagicly by type="filepicker"
    $('input[type=filepicker]', ctx).on('change', function (e) {
        e.preventDefault();
        var file = e.originalEvent.fpfile;
        $(this).val(JSON.stringify(file));
        $(this).parent().find('a').text(file.filename).attr('href', file.url);
    });
}


function refLink() {
    var $this = $(this); // jshint ignore:line
    var $a = $('<a class="btn" ></a>')
        .insertAfter($this)
        .click(function () {
            var id = $this.val() || 'new';
            var modelName = $this.data('modelname');
            var url = [window.root, '/model/', modelName, '/document/', id, '?_dialog=yes'].join('');
            window.showDialog($this, url);
        });

    $this.change(function () {
        $a.text($this.val() ? 'Edit' : 'New');
    });

    $this.change();
}


function ListField(el) {
    var self = this;
    self.el = $(el);

    self.add = function (e) {
        e.preventDefault();

        var li = $('<li />').hide()
            .append(self.template)
            .append(btn.delete())
            .append(btn.drag())
            [appendToFunc](self.list)
            .slideDown(function () {
                $('input:first', li).focus();
            });

        $('[name]', li).each(function () {
            var input = $(this),
                name = input.attr('name').replace(self.name + '_tmpl_', self.name + '_li' + self.length + '_');

            input.attr('name', name);
        });

        self.length++;

        // load nested widgets
        initWidgets(li);

        li.find('.nf_fieldset');
        li.find('> .nf_fieldset').click();

        li.find('> ul').sortable({
            items: 'li:not(.new_li)',
            handle: '.nf_listfield_drag'
        });
        $('.list_summary', self.el.parent()).text(self.length ? self.length + ' items' : 'No items');
    };

    if (self.el.data('processed') == 'true') {
        return;
    }
    self.el.data('processed', 'true');

    self.name = self.el.attr('name');

    var tpl = self.el.find('> .nf_hidden_template');
    tpl.find('.nf_listfield');
    self.template = tpl.html();
    tpl.remove();

    self.list = $('> ul', el);
    var appendFunc = (self.list.data('stack-dir') === 'top') ? 'prepend' : 'append';
    var appendToFunc = (self.list.data('stack-dir') === 'top') ? 'prependTo' : 'appendTo';

    self.el
        [appendFunc](btn.add().click(self.add))
        .append('<label class="list_summary" />');

    self.length = $('> li', self.list)
        .append(btn.drag())
        .append(btn.delete())
        .length;

    self.list.sortable({
        items: 'li:not(.new_li)',
        handle: '.nf_listfield_drag'
    });

    initWidgets(this);
}


function getQueryFunctionForSelect2() {
    var $this = $(this); // jshint ignore:line
    if (~$this[0].className.indexOf('select2')) return;
    var query_url = $this.data('url');
    var query_data = decodeURIComponent($this.data('data'));

    $this.select2({query: function (options) {
        var term = options.term;
        //var page = options.page;
        var context = options.context;
        var callback = options.callback;
        $.get(query_url, {
            data: query_data,
            query: term
        }).success(function (rsp) {
            var result = {
                results: rsp['objects'] || rsp,
                more: false,
                context: context
            };
            callback(result);
        });
    },
        initSelection: function (element, callback) {
            var id = $(element).val();
            if (id !== "") {
                $.get(query_url, {
                    data: query_data,
                    id: id
                }).done(function (rsp) {
                    callback(rsp);
                });
            }
        }
    });
}


function getSocketFunctionForSelect2() {
    var $this = $(this); // jshint ignore:line
    if (~$this[0].className.indexOf('select2')) return;
    var modelName = $this.data('modelname');

    $this.select2({
        query: _.debounce(function (options) {
            if (!window.socketio.connected) window.bootbox.alert("connectivity issues. best save, and refresh");
            if (_.isEmpty(options.term)) return options.callback({
                results: [],
                more: false,
                context: options.context
            });
            var term = options.term;
            //var page = options.page;
            var context = options.context;
            var callback = options.callback;
            window.socketio.json.emit(modelName, term, function (data) {
                var result = {
                    results: data,
                    more: false,
                    context: context
                };
                callback(result);
            });
        }, 600),
        initSelection: function (element, callback) {
            var id = $this.val();
            var text = $this.data('initialname') || '';
            callback({id: id, text: text});
        }
    }).on('change', function (e) {
        if (!e.added) return;
        window.socketio.json.emit('selected', e.added, function (id) {
            if (_.isEmpty(id)) return this.data([]);
            e.added.id = id;
        });
    });
}


function deleteDocument(callback) {
    $('#deleteButton').button('loading');
    var depsUrl = [window.root, 'json', 'model', window.model, 'document', window.docId, 'dependencies'].join('/');
    $.get(depsUrl).done(function (result) {
        var msg = result.length ? 'there are other entities dependent on this document:<ul><li>' + result.join('</li><li>') + '</li></ul>' : '';
        msg += 'Are you sure you want to delete?';
        window.bootbox.confirm(msg, function (res) {
            if (!res) return $('#deleteButton').button('reset');
            return $.ajax({
                type: 'POST',
                url: window.root + '/json/model/' + window.model + '/action/delete',
                data: {ids: [window.docId]},
                success: function () {
                    $('#deleteButton').button('reset');
                    if (callback) callback();
                },
                error: function (xhr, textStatus) {
                    $('#deleteButton').button('reset');
                    window.bootbox.alert(textStatus);
                    console.error('Deleting error', arguments);
                }
            });
        });
    });
}


function initActions() {
    $('button.action').click(function (e) {
        e.preventDefault();

        var action_id = $(this).val();
        if (!action_id) return;

        var ids = [window.docId];

        var msg = 'Are you sure you want to ' + $(this).text().toLowerCase() + ' this document? Changes made will not be saved!';

        window.bootbox.confirm(msg, function (result) {
            if (!result) return;

            $.post(window.root + '/json/model/' + window.model + '/action/' + action_id, { ids: ids }).always(function (data) {
                if (data.responseText) data = JSON.parse(data.responseText);
                if (data.error) {
                    window.bootbox.dialog("Some documents failed: " + data.error, [
                        {
                            "label": "Error",
                            "class": "btn-danger",
                            "callback": window.location.reload.bind(window.location)
                        }
                    ]);
                } else {
                    if (window.isDialog) {
                        window.dialogCallback({});
                    } else {
                        window.location.href = window.location.href.split('/document/')[0];
                    }
                }
            });
        });
    });

}


function initModal() {
    var modal = $('#myModal');
    window.showDialog = function (parentSelector, url) {
        modal.on('show', function () {
            modal.parentSelector = parentSelector;
            modal.find('h3').text(parentSelector.data('ref') + " Editor");
            modal.find('iframe').attr("src", url);
        });
        modal.modal('show');
    };

    window.hideDialog = function (response) {
        modal.response = response;
        modal.modal('hide');
    };

    modal.on('hide', function () {
        var response = modal.response;
        delete  modal.response;
        if (!response) return;
        if (response.cancel) return;
        // on delete
        if (response.delete) {
            modal.parentSelector.find("option[selected]").remove();
            modal.parentSelector.select2('val', '');
            modal.parentSelector.change();
            return;
        }
        // on create
        var id = modal.parentSelector.val();
        if (response.id && !id) {
            modal.parentSelector.find('option[selected]').removeAttr('selected');
            modal.parentSelector.append('<option selected value="' + response.id + '" >' + response.label + '</option>');
        }
        // on update
        if (response.id == id) {
            modal.parentSelector.find('option[value="' + id + '"]').text(response.label);
        }
        modal.parentSelector.select2('val', response.id);
        modal.parentSelector.change();
    });

    modal.on('hidden', function () {
        modal.find('iframe').attr("src", "about:blank");
        delete modal.parentSelector;
    });
}


if (window.Aviary) {
    //noinspection JSUnusedGlobalSymbols
    window.featherEditor = new Aviary.Feather({
        apiKey: window.AVIARY_API_KEY,
        apiVersion: 3,
        theme: 'dark', // Check out our new 'light' and 'dark' themes!
        tools: 'resize,crop,enhance',
        appendTo: '',
        onSave: function (imageID, newURL) {
            var $image = $(window.document.getElementById(imageID));
            var OrigInkBlob = $image[0].InkBlob;
            window.filepicker.writeUrl(OrigInkBlob, newURL, function (NewInkBlob) {
                $image.attr('src', NewInkBlob.url + '?' + Date.now());
                var $input = $image.parent().parent().find('input');
                $input.val(JSON.stringify(NewInkBlob));
                $input.parent().find('a.file-link').attr('href', NewInkBlob.url);
                window.featherEditor.close();
            });
        }
    });
}


function handle_file_picked($field, InkBlob) {
    var $image = $field.find('img.thumb-picker');
    $image.attr('src', InkBlob.url);
    $image[0].InkBlob = InkBlob;
    $field.find(".picker").text('Pick New File');
    $field.find(".editor").show();
    $field.find(".clearer").show();
    $field.find('input').val(JSON.stringify(InkBlob));
}


function handle_file_picked_aviary($field, InkBlob) {
    handle_file_picked($field, InkBlob);
    var $image = $field.find('img.thumb-picker');
    var sizeConstraint = $field.find("input").data('sizeconstraint');
    var args = {image: $image[0], url: InkBlob.url, initTool: 'crop'};
    if (sizeConstraint) {
        args.cropPresets = [
            ['Target', sizeConstraint],
            'Custom',
            'Original'
        ];
        args.cropPresetDefault = 'Target';
    }
    window.featherEditor.launch(args);
}


function trigger_picturepicker(e) {
    e.preventDefault();
    var $field = $(e.target).parent().parent();
    window.filepicker.pick({mimetype: 'image/*'}, handle_file_picked.bind(null, $field));
}


function trigger_aviarypicker(e) {
    e.preventDefault();
    var $field = $(e.target).parent().parent();
    window.filepicker.pick({mimetype: 'image/*'}, handle_file_picked_aviary.bind(null, $field));
}


function trigger_refilepicker(e) {
    e.preventDefault();
    var $field = $(e.target).parent().parent();
    var serInkBlob = $field.find('input').val();
    var InkBlob = JSON.parse(serInkBlob);
    handle_file_picked_aviary($field, InkBlob);
}


function trigger_clearpicker(e) {
    e.preventDefault();
    var $field = $(e.target).parent().parent();
    $field.find('input').val('');
    $field.find(".picker").text('Pick File');
    $field.find(".editor").hide();
    $field.find(".clearer").hide();
    $field.find('img.thumb-picker').attr('src', 'data:image/gif;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAAACADs%3D');
    $field.find('a.file-link').attr('href', '#');
}


$(function () {
    window.docId = window.location.pathname.split('/').pop();

    initWidgets();

    initModal();

    var $form = $('form#document');
    var $buttons = $form.find('p.submit button');
    var url = window.location.href.replace('/model', '/json/model');

    $form.submit(function (e) {
        e.preventDefault();
        var diag = window.bootbox.alert("Saving...");
        $buttons.prop('disabled', true);
        var q = $.post(url, $form.serialize())
            .done(function(data) {
                var urlParts = url.split('/document/');
                if (urlParts[1].slice(0,3) === 'new') {
                    var id = _.isString(data._id) ? data._id : data._id.$oid;
                    url = urlParts[0] + '/document/' + id;
                    window.history.pushState(data, id, url.replace('/json', ''));
                }
            })
            .fail(function (jqXHR) {
                var ret = jqXHR.responseJSON;
                window.bootbox.alert("Save failed - " + ret.name + '\n' + ret.fields);
            })
            .always(function () {
                setTimeout(function () {
                    diag.modal('hide');
                    $buttons.prop('disabled', false);
                }, 1000);
            });
        if (window.isDialog)
            q.done(function (data) { window.parent.hideDialog({id: data.id, label: data.label}); });
    });

    $form
        .on('click', 'button.picker', trigger_picturepicker)
        .on('click', 'button.aviary-picker', trigger_aviarypicker)
        .on('click', 'button.editor', trigger_refilepicker)
        .on('click', 'button.clearer', trigger_clearpicker);

    if (window.isDialog) {
        $('#cancelButton').click(function (e) {
            e.preventDefault();
            window.parent.hideDialog({cancel: true});
        });
    }

    $('#deleteButton').click(function () {
        deleteDocument(function () {
            if (window.isDialog) {
                window.parent.hideDialog({'delete': true});
            } else {
                window.location.href = window.location.href.split('/document/')[0];
            }
        });
    });

    initActions();

});

